﻿{
  このソースコードをひな形として新しくプラグインを開発する場合は、
  ・constのPluginMajorVersion～PluginCopyrightの書き換え
  ・RunProcess手続き
  ・RunPreference手続き(設定が必要な場合のみ)
  を実装すればOKです。
  また、このソースコードはMITライセンス下において自由に使用できます。

  MIT License

  Copyright (c) 2018 Lyna

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
}
unit PluginBase;

interface

const
  // プラグインのメジャーバージョン
  PluginMajorVersion = 1;
  // プラグインのマイナーバージョン
  PluginMinorVersion = 1;
  // プラグインの名称
  PluginName = 'Tatsuya Plugin';
  // プラグインの省略名(候補の右側に表示される)
  PluginShortName = 'TatsuyaPlugin';
  // プラグインの説明
  PluginDescription = '入力した文字に濁点を付けたりするATOKダイレクトプラグインです。';
  // プラグインのコピーライト
  PluginCopyright = '©Lyna(https://twitter.com/lynatan)';
  // Trueで"情報取得"対応(になるらしいが具体的に何なのか不明(環境設定の詳細情報を見ると"情報取得"と出る))
  PluginIsEnableText = False;
  // Trueで"自動検索"対応
  PluginIsCorrespondingAutoSearch = True;
  // Trueで"候補取得"対応(Falseでも候補は出るので用途不明)
  PluginIsEnableCandidate = True;

type
  TAlternativeType = (
    atNone = -1,       // 代替候補なし
    atAlternative = 0, // 代替候補文字列
    atOpenURL = 1      // 代替候補文字列をURLとして開く
  );

  TAddCandidateProc = reference to procedure(
    // 候補文字列
    const Candidate: string;
    // 候補に対応するコメント(XHTML)
    const Comment: string = '';
    // 代替候補文字列はAlternativeTypeの内容によって扱いが変わる
    //   atNone       : 代替候補は使用されない
    //   atAlternative: Shift+Enterで代替候補が確定文字列となる
    //   atOpenURL    : Shift+Enterで代替候補をURLとして開く
    const Alternative: string = '';
    // 代替候補として実際にツールチップに表示される文字列(未指定の場合はAlternativeが直接表示される)
    const AlternativeAlias: string = '';
    // 代替候補タイプ
    AlternativeType: TAlternativeType = atNone;
    // 候補ウィンドウで候補文字列の右側に表示される文字列
    const Tag: string = ''
  );

procedure RunProcess(const InputText: string; const AddCandidateProc: TAddCandidateProc);
procedure RunPreference;

implementation

uses
  Winapi.Windows;

// 入力文字列に対して候補となる文字列をAddCandidateProcで追加する
// 何もしなければ候補なしとして処理される
procedure RunProcess(const InputText: string; const AddCandidateProc: TAddCandidateProc);
const
  XmlHeader = '<?xml version="1.0" encoding="UTF-8" ?>' +
              '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja" lang="ja">' +
              '<head><title>AtokDirect</title></head>' +
              '<body>';
  XmlFooter = '</body>' +
              '</html>';
  TatsuyaProfile = '<b>藤原 竜也</b>（ふじわら たつや、1982年5月15日 - ）は、日本の俳優。<br/>' +
                   '埼玉県秩父市出身。ホリプロ所属。秩父市立秩父第二中学校卒業。堀越高等学校中退。<br/>' +
                   '<a href="https://ja.wikipedia.org/wiki/%E8%97%A4%E5%8E%9F%E7%AB%9C%E4%B9%9F">詳細をWikipediaで見る</a>';

  function TatsuyaizeText(const Text: string): string;
  var
    c: Char;
  begin
    Result := '';
    for c in Text do
      Result := Result + c + '゛';
  end;

  function SuddenDeathText(const Text: string): string;
  var
    len: Integer;
    c: Char;

    function RepeatStr(const S: string; Count: Integer): string;
    begin
      Result := '';
      while Count > 0 do
      begin
        Result := Result + S;
        Dec(Count);
      end;
    end;

  begin
    len := 0;
    for c in Text do
      Inc(len, Ord((Word(c) < $20) or (Word(c) > $7E)) + 1); // 雑半角判定
    len := len div 2;
    Result := '＿' + RepeatStr('人', len + 2) + '＿'#13#10 +
              '＞　' + Text + '　＜'#13#10 +
              ' ￣Y' + RepeatStr('^Y', len) + '￣';
  end;

begin
  AddCandidateProc(TatsuyaizeText(InputText), XmlHeader + TatsuyaProfile + XmlFooter);
  AddCandidateProc(SuddenDeathText(InputText));
  AddCandidateProc(SuddenDeathText(TatsuyaizeText(InputText)));
  {
  // 代替候補でGoogle検索するサンプル(System.NetEncodingが必要)
  s := 'https://www.google.co.jp/search?q=' + TNetEncoding.URL.Encode(InputText);
  AddCandidateProc(s, '', s, 'Googleで検索: ' + InputText, atOpenURL, 'URL');
  }
end;

procedure RunPreference;
begin
  // 環境設定の設定ボタン押下時に呼ばれる(設定がある場合はここで設定ダイアログを呼び出す)
  // 普通にTFormを継承したフォームをShowModalで呼び出せばOK
  // 表示位置はGetActiveWindow->GetWindowRectで親ウィンドウの位置を取って適当に調整
  MessageBox(GetActiveWindow, '設゛定゛な゛ん゛て゛な゛い゛よ゛お゛お゛お゛！゛！゛！゛', PluginName, MB_OK);
end;

end.